/*************************************************************************
 * FILE:	fir.c
 *
 * COPYRIGHT:	(C) 2011 Atmel Corp. <www.atmel.com>
 *
 * ABSTRACT:	FIR filter example
 *
 * DESCRIPTION: This simple example filters a stored input stream and places
 *              the output stream in RAM;
 *
 * MODIFICATION HISTORY:
 * 2011/05/27 : created	for Keil C51
 *
 ************************************************************************/

#include ".\FIR.h"
#include <AT89LP6440.h>
#include <stdlib.h>


#define N_TAPS 24		// define the filter order here; must be <= 128
#define N_SFT 1			// # of shifts required to normalize, minus 1
#define N_SAMPLES 64

sfr FIRD = 0xE3 ;
sfr DSPR = 0xE2 ;

#include "lib_fir.h"

int get_sample(unsigned char i);

/* The FIFO can only be located at 0x0000 or 0x0100 
 */
  int xdata FIFO[2*N_TAPS] _at_ 0x0000 ;

/* filter cofficients in reverse order (b23->b0)
 * # coeffs = N_TAPS
 */
  int code COEFF[] = { 0x01ba, 0x07a5, 0x0569, 0x014d, 0xf7d8, 0xee8c, 0xec7c, 0xf882,
	                   0x1486, 0x3b16, 0x6083, 0x7792, 0x7792, 0x6083, 0x3b16, 0x1486,
	                   0xf882, 0xec7c, 0xee8c, 0xf7d8, 0x014d, 0x0569, 0x07a5, 0x01ba,
					   N_SFT*256 };	// <-- place # of shifts in HIGH byte here

void main(void){

  int xdata *fifoP ;
  int code *coeffP;
  int sample, result;
  unsigned char i;
  int xdata SAVE[N_SAMPLES];

					   	
 
//------------------------------------------------------------------------
// FIR Filter Setup
//------------------------------------------------------------------------
 
  coeffP = COEFF;
  fifoP = FIFO;

  // clear fifo
  clear_fifo(fifoP, N_TAPS);
    
  // set fifo depth
  FIRD = 2*N_TAPS-1;
  DSPR = 0x84;		// MRW1, XPSQ0  set


//------------------------------------------------------------------------
// FIR Filter Computation
//------------------------------------------------------------------------

  for (i=0; i<N_SAMPLES; i++)
  { // get sample 
    sample = get_sample(i);
    // and store in fifo
    fifoP = store_in_fifo(fifoP, sample);
    // compute output 
    result = compute_fir(fifoP, coeffP, N_TAPS);
	// and store in buffer
	SAVE[i] = result;
  }
	
  while(1);

}

int get_sample(unsigned char i)
{ /* in a real application this would fetch the next sample
   * from the ADC or a communication interface
   */
  int code wave[] = { 0x0000, 0x4c7b, 0x187d, 0xe38e, 0x2d40, 0x7535, 0x3b20, 0xfec5, 
	                  0x3fff, 0x7ec4, 0x3b20, 0xf536, 0x2d40, 0x638d, 0x187e, 0xcc7c, 
	                  0xffff, 0x3383, 0xe782, 0x9c72, 0xd2be, 0x0ac9, 0xc4e0, 0x813b,
	                  0xbfff, 0x013a, 0xc4e0, 0x8aca, 0xd2be, 0x1c71, 0xe782, 0xb384,
	                  0xffff, 0x4c7b, 0x187e, 0xe38e, 0x2d40, 0x7535, 0x3b21, 0xfec5,
	                  0x3ffe, 0x7ec4, 0x3b21, 0xf536, 0x2d40, 0x638d, 0x187e, 0xcc7c,
	                  0xffff, 0x3383, 0xe783, 0x9c72, 0xd2be, 0x0ac9, 0xc4e0, 0x813b,
	                  0xbfff, 0x013a, 0xc4e0, 0x8aca, 0xd2bd, 0x1c71, 0xe783, 0xb384 };
  if (i < N_SAMPLES)
    return(wave[i]);
  else
    return(0);
}
