/*************************************************************************
 * FILE:	iir.c
 *
 * COPYRIGHT:	(C) 2011 Atmel Corp. <www.atmel.com>
 *
 * ABSTRACT:	IIR filer example
 *
 * DESCRIPTION:	This simple example filters a stored input stream and places
 *              the output stream in RAM;
 *              NOTE: Keil is big endian
 *
 * MODIFICATION HISTORY:
 * 2011/05/26 : created for Keil C51
 *
 ************************************************************************/

#include ".\IIR.h"
#include <AT89LP6440.h>
#include <stdlib.h>


#define N_TAPS 6		// define the filter order here; must be <= 128
#define N_ASFT 4		// # of shifts required to normalize A
#define N_BSFT 12		// # of shifts required to normalize B
#define N_SAMPLES 64

sfr FIRD = 0xE3 ;
sfr DSPR = 0xE2 ;

#include "lib_iir.h"

int get_sample(unsigned char i);

/* The FIFO can only be located at 0x0000 or 0x0100 
 */
  int xdata FIFO_A[2*(N_TAPS-1)] _at_ 0x0000 ;
  int xdata FIFO_B[2*N_TAPS] _at_ 0x0100 ;


/* filter cofficients in reverse order (a5->a1,b5->b0)
 * # coeffs = N_TAPS
 */
  int code COEFF_A[] = { 0x0a50, 0xc9ea, 0x73aa, 0x81e0, 0x4639,
 			 N_ASFT*256 };	// <-- place # of shifts in HIGH byte here 
  int code COEFF_B[] = { 0x3df7, 0xb34d, 0x3967, 0x3967, 0xb34d, 0x3df7,
 			 N_BSFT*256 };	// <-- place # of shifts in HIGH byte here


void main(void){

  int xdata *fifoA_P, *fifoB_P ;
  int code *coeffA_P, *coeffB_P ;
  int sample, result;
  unsigned char i;
  int xdata SAVE[N_SAMPLES];
	
//------------------------------------------------------------------------
// FIR Filter Setup
//------------------------------------------------------------------------

  coeffA_P = COEFF_A;
  fifoA_P = FIFO_A;
  coeffB_P = COEFF_B;
  fifoB_P = FIFO_B;

  // clear fifos 
  clear_fifo(fifoA_P, N_TAPS-1);
  clear_fifo(fifoB_P, N_TAPS);
    
  // configure MAC, FIFOs
  DSPR = 0x4C;		// set MRW0, XPSQ1, XSPQ0
 
//------------------------------------------------------------------------
// FIR Filter Computation
//------------------------------------------------------------------------

  for (i=0; i<N_SAMPLES; i++)
  { // get sample 
    sample = get_sample(i);
    // set fifo depth
    FIRD = (2*N_TAPS)-1;
    // and store in fifo
    fifoB_P = store_in_fifo(fifoB_P, sample);
    // compute intermediate 
    compute_iir_B(fifoB_P, coeffB_P, N_TAPS);
    // set fifo depth
    FIRD = (2*(N_TAPS-1))-1;
    // compute output 
    result = compute_iir_A(fifoA_P, coeffA_P, N_TAPS-1);
    // and store in fifo
    fifoA_P = store_in_fifo(fifoA_P, result);
    // and store in buffer
    SAVE[i] = result;
  }
	
  while(1);
}

int get_sample(unsigned char i)
{ /* in a real application this would fetch the next sample
   * from the ADC or a communication interface
   */
  int code wave[] = { 0x0000, 0x4c7b, 0x187d, 0xe38e, 0x2d40, 0x7535, 0x3b20, 0xfec5, 
	              0x3fff, 0x7ec4, 0x3b20, 0xf536, 0x2d40, 0x638d, 0x187e, 0xcc7c, 
	              0xffff, 0x3383, 0xe782, 0x9c72, 0xd2be, 0x0ac9, 0xc4e0, 0x813b,
	              0xbfff, 0x013a, 0xc4e0, 0x8aca, 0xd2be, 0x1c71, 0xe782, 0xb384,
	              0xffff, 0x4c7b, 0x187e, 0xe38e, 0x2d40, 0x7535, 0x3b21, 0xfec5,
	              0x3ffe, 0x7ec4, 0x3b21, 0xf536, 0x2d40, 0x638d, 0x187e, 0xcc7c,
	              0xffff, 0x3383, 0xe783, 0x9c72, 0xd2be, 0x0ac9, 0xc4e0, 0x813b,
	              0xbfff, 0x013a, 0xc4e0, 0x8aca, 0xd2bd, 0x1c71, 0xe783, 0xb384 };
  if (i < N_SAMPLES)
    return(wave[i]);
  else
    return(0);
}
